@@ -17,3 +17,5 @@ pub mod recipients;
 pub mod remove;
 pub mod show;
 pub mod sync;
+#[cfg(all(feature = "tomb", target_os = "linux"))]
+pub mod tomb;
